<?php
//
// +---------------------------------------------------------------------------+
// | Glossary Plugin for Geeklog - The Ultimate Weblog                         |
// +---------------------------------------------------------------------------+
// | functions.inc                                                             |
// | This file does two things: 1) it implements the necessary Geeklog Plugin  |
// | API method and 2) implements all the common code needed by the Glossary   |
// | PHP files.                                                                |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2003 by the following authors:                              |
// | Alexander Schmacks          -    alexander@schmacks.de                    |
// | Copyright (C) 2021 by the following authors:                              |
// | Remy Kobolski               -    remy@nlbox.com                           |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+
//

/**
* This is the generic functions.inc for the Universal Plugin Toolkit for Geeklog.
* You need to change all instances of {plugin} with the name of your plugin.
*/

/**
* Language file Include
*/
$langfile = $_CONF['path'] . 'plugins/glossary/language/' . $_CONF['language'] . '.php';

if (file_exists ($langfile)) {
    include_once ($langfile);
} else {
    include_once ($_CONF['path'] . 'plugins/glossary/language/english.php');
}

/*
* Include Glossary config file
*/
require_once ($_CONF['path'] . 'plugins/glossary/config.php');

// +---------------------------------------------------------------------------+
// | Geeklog Plugin API Implementations                                        |
// +---------------------------------------------------------------------------+

/**
 * 
 * Geeklog is asking us to provide any new items that show up in the type drop-down
 * on search.php.  Let's let users search the entries in the glossary
 *
 *  @author: uy
 **/
function plugin_searchtypes_glossary() {
    global $LANG_GLS_00;
    $lReturnTypes['glossary'] = $LANG_GLS_00['searchlabel'];
    return $lReturnTypes;
}


/**
 * This searches for glossary entries matching the user query and returns an array
 * of header and table rows back to search.php where it will be formated and
 * printed
 *
 * @param query            string          Keywords user is looking for
 * @param datestart        date/time       Start date to get results for
 * @param dateend          date/time       End date to get results for
 * @param topic            string          The topic they were searching in
 * @param type             string          Type of items they are searching
 * @param author           string          Get all results by this author
 *
 * @author: uy
 **/
function plugin_dopluginsearch_glossary($pQuery, $pDatestart, $pDateend, $pTopic, $pType, $pAuthor)
{
    global $_TABLES, $_CONF, $LANG_GLS_00;

    if (empty($pType)) {
        $pType = 'all';
    }
   
    // Bail if we aren't supppose to do our search
    if ($pType <> 'all' AND $pType <> 'glossary') {
        $lPluginResults = new Plugin();
        $lPluginResults->plugin_name = 'glossary';
        $lPluginResults->searchlabel = $LANG_GLS_00['searchlabel'];
        return $lPluginResults;
    }

    // Build search SQL for glossary entries according to the given parameters
    $lSQL = "SELECT entries.id, abbreviation, explanation, owner, timestamp, "
           ."categories.id as category_id, categories.value as category_name, "
           ."projects.id as project_id, projects.value as project_name "
           ."FROM ({$_TABLES['gls_entry']}        AS entries) "
           ."LEFT JOIN {$_TABLES['gls_category']} AS categories ON (categories.id = entries.category) "
           ."LEFT JOIN {$_TABLES['gls_project']}  AS projects   ON (projects.id = entries.project) "
           ."WHERE (explanation LIKE '%{$pQuery}%' OR abbreviation LIKE '%{$pQuery}%') ";

    // If date start and end are given, add the appropriate criterion
    if (!empty($pDatestart) && !empty($pDateend)) {
        $delim = $pDatestart[4];
        $DS = explode($delim,$pDatestart);
        $DE = explode($delim,$pDateend);
        $startdate = mktime(0,0,0,$DS[1],$DS[2],$DS[0]);
        $enddate = mktime(0,0,0,$DE[1],$DE[2],$DE[0]) + 3600;
        $lSQL .= "AND (UNIX_TIMESTAMP(timestamp) BETWEEN '$startdate' AND '$enddate') ";
    } elseif (!empty($pDatestart)) {
        $delim = $pDatestart[4];
        $DS = explode($delim,$pDatestart);
        $startdate = mktime(0,0,0,$DS[1],$DS[2],$DS[0]);
        $lSQL .= "AND (UNIX_TIMESTAMP(timestamp) >= '$startdate') ";
    } elseif (!empty($pDateend)) {
        $delim = $pDateend[4];
        $DE = explode($delim,$pDateend);
        $enddate = mktime(0,0,0,$DE[1],$DE[2],$DE[0]) + 3600;
        $lSQL .= "AND (UNIX_TIMESTAMP(timestamp) <= '$enddate') ";
    }
    
    // If author is given, add him to the criterion
    if ( $pAuthor > 0 ) {
        $lSQL .= "AND (owner = $pAuthor) ";
    }

    $lResult = DB_query($lSQL);
    $lResultCount = DB_numRows($lResult);

    // OK, now create new plugin object and insert table header labels
    require_once($_CONF['path_system'] . 'classes/plugin.class.php');
    $lPluginResults = new Plugin();
    $lPluginResults->plugin_name = 'glossary';
    $lPluginResults->searchlabel = $LANG_GLS_00['searchlabel'];
    $lPluginResults->addSearchHeading('Title');
    $lPluginResults->addSearchHeading('Description');
    $lPluginResults->addSearchHeading('Date');
    $lPluginResults->addSearchHeading('Author');

    // Build result lines
    for ($i = 1; $i <= $lResultCount; $i++) {
        $A = DB_fetchArray($lResult);
        $description = GLS_truncate($A['explanation']);
        // Right now we don't have security for single glossary entries
        //if(SEC_hasAccess($A[owner_id],$A[group_id],$A[perm_owner],$A[perm_group],$A[perm_members],$A[perm_anon])){
            $rcnt++;
            $bDate = strftime($_CONF['date'],strtotime($A['timestamp']));
            $row = array( '<a href="'. $_CONF['site_url'] .'/glossary/entry.php?mode=view&id='. $A['id'] .'">'. $A['abbreviation']  .'</a>', 
                          '['.$A['project_name'] .'/'. $A['category_name'] .'] '. $description,
                          $bDate,
                          '<a href="'. $_CONF['site_url'] .'/users.php?mode=profile&uid='. $A['owner'] .'">'
                                     . DB_getItem($_TABLES['users'],'username',"uid = '{$A['owner']}'") .'</a>');
            $lPluginResults->addSearchResult($row);
        //}
    }
    $lPluginResults->num_searchresults = $rcnt;
    $lPluginResults->num_itemssearched = DB_count($_TABLES['gls_entry']);
    
    return $lPluginResults;
}



/**
* Returns the items for this plugin that should appear on the main menu
*
* NOTE: this MUST return the url/value pairs in the following format
* $<arrayname>[<label>] = <url>
*
* Universal Plugin:  Uncomment and fill in variables;
* 
* @return    array   Associative array of menu items to be added to menu
*
*/
function plugin_getmenuitems_glossary()
{
    global $_CONF, $LANG_GLS_00, $_GLS_CONF;

    $lMenuLabel = $LANG_GLS_00['shortname'];                
    $lMenuURL = $_CONF['site_url'] . "/glossary/index.php"; 
    
    $lMenuItems = array();
    if (SEC_hasRights('glossary.view') || $_GLS_CONF['show_to_anyone'] ) {
        $lMenuItems["$lMenuLabel"] = $lMenuURL;
    }
    return $lMenuItems;

}

/**
* Check to see if this Plugin uses comments
*
* Geeklog is checking to see if this plugin supports comments, tell it no!
* NOTE: to support comments you must use the same date/time based ID for your
* widget.  In other words, to make primary keys for your plugin you should call
* makesid().  Comments are keyed off of that...it is a limitation on how geeklog
* does comments.
*
* @return   boolean true or false
*
*/
function plugin_commentsupport_glossary() 
{
    return false;
}

/**
* Shows the statistics for the plugin on stats.php.  If
* $showsitestats is 1 then we are to only print the overall stats in the 'site
* statistics' box otherwise we show the detailed stats for the plugin
*
* Universal Plugin:  Modify/uncomment use it
*
* @param    int showsitestate   Flag to let us know which stats to get
* @return   string  returns formatted HTML to insert in stats page
*
*/
function plugin_showstats_glossary($showsitestats) 
{
    global $_CONF, $_TABLES;

    $retval='';
    if ($showsitestats == 1) {
        // This shows in the summary box
        $total_pages = DB_count($_TABLES['gls_entry']);  // Fill this with count for summary
        $summary_label = "Glossary";                     // Fill this with label displayed
        
        $retval  = "<table border = '0' width='100%' cellspacing='0' cellpadding='0'>";
        $retval .= "<tr><td>$summary_label</td>";
        $retval .= "<td align='right'>" . $total_pages . "&nbsp;&nbsp </td></tr></table>";
    } else {
        $stat_templates = new Template($_CONF['path_layout'] . 'stats');
        $stat_templates->set_file(array('itemstats' => 'itemstatistics.thtml',
                                    'statrow'   => 'singlestat.thtml'));
        // This fills the top the box  Use the following as a template
//      $result = DB_query("SELECT id, abbreviation as 'title', hits from {$_TABLES['gls_stats'] LEFT JOIN  {$_TABLES['gls_entry'] ON id = id WHERE hits > 0 ORDER BY hits desc LIMIT 10");
//      $nrows  = DB_numRows($result);
        $nrows = 0;
        $retval .= COM_startBlock("Top Ten Glossary Entries");
        if ($nrows > 0) {
            $stat_templates->set_var('item_label',"Abbreviation");
            $stat_templates->set_var('stat_name',"Hits");
            for ($i = 0; $i < $nrows; $i++) {
                $A  = DB_fetchArray($result);
                $stat_templates->set_var('item_url', $_CONF['site_url'] . '/glossary/index.php?id=' . $A['id']);
                $stat_templates->set_var('item_text', $A['title']);
                $stat_templates->set_var('item_stat', $A['hits']);
                $stat_templates->parse('stat_row','statrow',true); 
            }
            $stat_templates->parse('output','itemstats');
            $retval .= $stat_templates->finish($stat_templates->get_var('output'));
        } else {
            $retval .= 'It appears that there are no Glossarys on this site or no one has ever viewed them.';
        }    
        $retval .= COM_endBlock();
    }
    return $retval;
}
/**
* New stats plugin API function for proper integration with the site stats
*
* @return   array(item text, item count);
*
*/
function plugin_statssummary_glossary ()
{
    global $_TABLES;
    // This shows in the summary box
    $total_pages   = DB_count($_TABLES['gls_entry']);  // Fill this with count for summary
    $summary_label = "Glossary entries";             // Fill this with label displayed
    return array ($summary_label, $total_pages);
}


/**
* This will put an option for the plugin in the command and control block on moderation.php
*
* Add the plugin name, icon and link to the command and control block in moderation.php
*
* @return   array   Array containing (plugin name, admin url, url of plugin icon)
*
*/
function plugin_cclabel_glossary() 
{
    global $_CONF;
    return array('glossary',$_CONF['site_admin_url'] . '/plugins/glossary/index.php',$_CONF['site_url'] . '/glossary/images/Glossary.gif');
}

/**
* Returns the administrative option for this plugin
*
* Adds the plugin to the Admin menu
*
* Universal Plugin:  Modify/Uncomment use it
*
* @return   array   Array containing (plugin name, plugin admin url, # of items in plugin or '')
*
*/
function plugin_getadminoption_glossary() 
{
    global $_CONF;
    if (SEC_hasRights('glossary.admin')) {
        return array('Glossary', $_CONF['site_admin_url'] . '/plugins/glossary/index.php', 0);
    }
    return false;
}

/**
* Returns the user option for this plugin
*
* Adds the plugin to the user menu
*
* Universal Plugin:  modify/uncomment use it
*
* @return   array   Array containing (plugin name, plugin user url, 0)
*
*/
function plugin_getuseroption_glossary() 
{
    global $_CONF, $LANG_GLS_00;
    if (SEC_hasRights('glossary.edit,glossary.view','OR')) {
        return array($LANG_GLS_00['shortname'], $_CONF['site_url'] . '/glossary/index.php', 0);
    }
    return false;
}

function plugin_whatsnewsupported_glossary()
{
    global $_GLS_CONF, $LANG_GLS_00, $LANG_WHATSNEW;

    if ( $_GLS_CONF['hide_whatsnew'] != 'hide' ) {
        $retval = array( $LANG_GLS_00['shortname'], COM_formatTimeString( $LANG_WHATSNEW['new_last'], $_GLS_CONF['whatsnew_interval'] ));
    } else {
        $retval = false;
    }
    return $retval;
}

function plugin_getwhatsnew_glossary()
{
    global $_CONF;
    global $_TABLES;
    global $_GLS_CONF;

    $retval = '';

    if ($_GLS_CONF['hide_whatsnew'] == 'hide') {
        return $retval;
    }

    $select  = "SELECT id, abbreviation FROM {$_TABLES['gls_entry']} ";
    $where   = "WHERE (last_timestamp >= (DATE_SUB(NOW(), INTERVAL {$_GLS_CONF['whatsnew_interval']} SECOND)))" ;
    $orderBy = "ORDER BY last_timestamp DESC LIMIT 15";

    $sql     = $select .' '. $where .' '. $orderBy;
    $result  = DB_query($sql);
    $nrows   = DB_numRows($result);

    if ($nrows > 0) {
        $recent = array();
        for ($x = 0; $x < $nrows; $x++) {
            $A        = DB_fetchArray($result, false);
            $name     = $A['abbreviation'];
            $url      = COM_buildUrl($_CONF['site_url'] . '/glossary/entry.php?mode=view&id=' . $A['id']);
            $recent[] = COM_createLink($name, $url);
        }
        $retval .= COM_makeList($recent, 'list-new-plugins');
    } else {
        $retval .= $LANG_GLS_00['Norecentnew'] . '<br' . XHTML . '>' . "\n";
    }
    return $retval;
}

function plugin_autotags_glossary ($option, $content = '', $autotag = '')
{
    global $_CONF, $_GLS_CONF, $_TABLES;
    $comment = '';
    switch ($option) {
        case 'tagname' :
            return 'glossary';
            break;
        case 'nopermission':
            return '';
            break;
        case 'permission'  :
            return 'glossary';
            break;
        case 'closetag'  :
            return [];
            break;
        case 'description':
            return 'Glossary of Terms';
            break;
        case 'parse':
            if (empty($_GLS_CONF['comment'])) {
                $comment = '';
            } else {
                $comment = '&nbsp;<img src="'.$_CONF['site_url'].'/glossary/images/Hint.png" border="0" title="'
                         . sprintf($_GLS_CONF['comment'], $autotag['parm1'])
                         .'" >&nbsp;';
            } 
            $content = str_replace($autotag['tagstr'], $comment, $content);
            if ($autotag['parm1'] == 'All') {
                $project = 0;
            } else {
                $project = DB_getItem($_TABLES['gls_project'], 'id', "value = '{$autotag['parm1']}' ");
            }
            if ($project !== false) {
                $content = GLS_insertGlossaryLinks($content, '', $project);
            }
            break;
    }
    return $content;
}

    
if ($op == 'tagname' ) {
        return 'glossary';
    } elseif ($op == 'parse') {
    return $content;
}

function plugin_user_delete_glossary($uid)
{
    global $_TABLES;
    global $_GLS_CONF;

    if (DB_count ($_TABLES['gls_entry'], 'owner', $uid) > 0) {
        $sql      = "UPDATE {$_TABLES['gls_entry']} SET owner = 2 WHERE owner = $uid";
        $result   = DB_query ($sql);
    }
    if (DB_count ($_TABLES['gls_entry'], 'last_owner', $uid) > 0) {
        $sql      = "UPDATE {$_TABLES['gls_entry']} SET last_owner = 1 WHERE last_owner = $uid";
        $result   = DB_query ($sql);
    }
}

/**
* Removes the datastructures for this plugin from the Geeklog database.
* This routine will get called from the Plugin install program if user select De-Install or if Delete is used in the Plugin Editor.
* The Plugin Installer will also call this routine upon and install error to remove anything it has created.
* The Plugin installer will pass the optional parameter which will then double check that plugin has first been disabled. 
* 
* Returns True if all Plugin related data is removed without error
*
* @param    string   $installCheck     Defult is blank but if set, check if plugin is disabled first
* @return   boolean True if successful false otherwise
*
*/  
function plugin_uninstall_glossary($installCheck='')
{
    global $_TABLES, $LANG_GLS_00;

    // The Glossarys need to be changed to the name of your plugin
    // Enter the names of your plugin table names for table1 ...
    $pi_name  = 'glossary';
    $FEATURES = array ( 'glossary.admin', 'glossary.view', 'glossary.edit');
    $TABLES   = array ('gls_entry','gls_category','gls_project');
    
    // Check and see if plugin is still enabled - if so display warning and exit
    if (DB_getItem($_TABLES['plugins'],'pi_enabled', 'pi_name = "' .$pi_name. '"')) {
        GLS_errorLog("Plugin is installed and enabled. Disable first if you want to de-install it",1);
        $display .= COM_startBlock($LANG_GLS_00['warning']);
        $display .= $LANG_GLS_00['enabled'];
        $display .= COM_endBlock();
        $display  = GLS_render($display, 'all');
        COM_output($display);
        exit;
    }
           
    // Ok to proceed and delete plugin - Unregister the plugin with Geeklog
    GLS_errorLog('Attempting to unregister the Glossary plugin from Geeklog',1);
    DB_query("DELETE FROM {$_TABLES['plugins']} WHERE pi_name = 'glossary'",1);

    // Drop Glossary tables
    foreach($TABLES as $table) {
        $t = $_TABLES["$table"];
        GLS_errorLog("Removing Table $t",1);
        DB_query("DROP TABLE $t",1);
    }

    // Remove Security for this plugin
    $grp_id = DB_getItem($_TABLES['vars'], 'value', "name = '{$pi_name}_gid'");
    
    GLS_errorLog("Removing $pi_name Admin Group", 1);
    DB_query("DELETE FROM {$_TABLES['groups']} WHERE grp_id = $grp_id",1);
    DB_query("DELETE FROM {$_TABLES['vars']} WHERE name = '{$pi_name}_gid'");
    GLS_errorLog("Removing root users from admin of $pi_name");
    DB_query("DELETE FROM {$_TABLES['group_assignments']} WHERE ug_main_grp_id = $grp_id",1);
    
    // Remove all the associated features -- access rights
    foreach ($FEATURES as $feature) {
        GLS_errorLog("Removing $feature feature and rights to it",1);
        $feat_id = DB_getItem($_TABLES['features'], 'ft_id', "ft_name = '$feature'");
        DB_query("DELETE FROM {$_TABLES['access']} WHERE acc_ft_id = $feat_id",1);
        DB_query("DELETE FROM {$_TABLES['features']} WHERE ft_id = $feat_id",1);
    }
        
    GLS_errorLog('...success',1);
    return true;
}
 ?>