<?php
//
// +---------------------------------------------------------------------------+
// | Glossary Plugin for Geeklog - The Ultimate Weblog                         |
// +---------------------------------------------------------------------------+
// | functions.inc                                                             |
// | This file does two things: 1) it implements the necessary Geeklog Plugin  |
// | API method and 2) implements all the common code needed by the Glossary   |
// | PHP files.                                                                |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2003 by the following authors:                              |
// | Alexander Schmacks          -    alexander@schmacks.de                    |
// | Copyright (C) 2021 by the following authors:                              |
// | Remy Kobolski               -    remy@nlbox.com                           |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+
//

require_once ($_CONF['path'] . 'plugins/glossary/api.inc');

/**
 * Returns true of false for correct spelling of Abbreviation: letter, digit, underscore, space of hyphen
 *                       not starting with whiteSpace, digit or hyphen
 *                       not ending with whiteSpace or hyphen
 **/ 
function GLS_isValidAbbreviation($tag)
{
    if (is_numeric($tag) || $tag[0] == '-' || $tag[0] == ' ' ) {
        return false;
    }
    $x = strlen($tag)-1;
    if ($tag[$x] == '-' || $tag[$x] == ' ') {
        return false;
    }
    if (strlen($tag) > 2 || $tag === strtoupper($tag)) {
        return preg_match('/^[\h\w\-]+$/', $tag);
    } else {
        return false;
    }
}

function GLS_assembleWhereClause($selection)
{
    $lSelection = GLS_assembleSelection($selection, true);
    if ( empty($lSelection)) {
        return '';
    } else {
        return 'WHERE ' . join(' AND ', $lSelection);
    }
}
function GLS_assembleUrlParameters($selection, $emptyFlag = false)
{
    $lSelection = GLS_assembleSelection($selection, false);
    if ($emptyFlag) {
        $prefix = '';  $suffix = '';
    } else {
        $prefix = '?'; $suffix = '&';
    }
    if ( empty($lSelection)) {
        return $prefix;
    } else {
        return '?' . join('&', $lSelection) . $suffix;
    }
}
function GLS_assembleSelection($selection, $sqlFlag = true)
{
    $pCategory     = $selection['category'];
    $pProject      = $selection['project'];
    $pTimeLimit    = $selection['timelimit'];
    $pLetter       = $selection['letter'];
    $pAbbreviation = $selection['abbreviation'];
    $pContributed  = $selection['contributed'];
    $pAddedDuring  = $selection['addedDuring'];

    $lWhereClause = [];
    if ( !empty($pCategory) ) {
        $lWhereClause[] = "category={$pCategory}";
    }
    if ( !empty($pProject) ) {
        $lWhereClause[] = "project={$pProject}";
    }
    if ( !empty($pTimeLimit) ) {
        $lWhereClause[] = $pTimeLimit;
    }
    if ( !empty($pLetter) ) {
        if ($sqlFlag) {
            $lWhereClause[] = "letter='{$pLetter}'";
        } else {
            $lWhereClause[] = "letter={$pLetter}";
        }
    }
    if ( !empty($pAbbreviation) ) {
        if ($sqlFlag) {
            $lWhereClause[] = "abbreviation='{$pAbbreviation}'";
        } else {
            $lWhereClause[] = "abbreviation={$pAbbreviation}";
        }
    }
    if ( !empty($pContributed) && SEC_hasRights('glossary.edit,glossary.admin','OR') ) {
        if ($sqlFlag) {
            $lWhereClause[] = "(owner = $pContributed OR last_owner = $pContributed)";
        } else {
            $lWhereClause[] = "contributed={$pContributed}";
        }
    }
    if ( !empty($pAddedDuring) ) {
        $lWhereClause[] = "addedDuring={$pAddedDuring}";
    }
    return $lWhereClause;
}

/**
 * Returns an array of all projects for the glossary.
 **/
function GLS_getProjects($selection = array())
{
    global $_TABLES;
    
    $retval = array();

    $lWhereClause = GLS_assembleWhereClause($selection);

    $result = DB_query("SELECT id, value from {$_TABLES['gls_project']} ORDER BY id asc");
//  $result = DB_query("SELECT DISTINCT(project), p.id, value from {$_TABLES['gls_entry']} LEFT JOIN {$_TABLES['gls_project']} AS p ON (project = p.id) $lWhereClause ORDER BY id asc");
    $nrows  = DB_numRows($result);
    for ($i = 1; $i<=$nrows; $i++) {
        $bArray = DB_fetchArray($result);
        $retval[$bArray['id']] = $bArray['value'];
    }
        
    return $retval;
}

/**
 * Returns an array of all projects for the glossary.
 **/
function GLS_getCategories($selection = array())
{
    global $_TABLES;
    
    $retval = array();
    
    $result = DB_query("SELECT id, value from {$_TABLES['gls_category']} ORDER BY id asc");
    $nrows  = DB_numRows($result);
    for ($i = 1; $i<=$nrows; $i++) {
        $bArray = DB_fetchArray($result);
        $retval[$bArray['id']] = $bArray['value'];
    }
        
    return $retval;
}

/**
 * Returns an array of all owners of the glossary.
 **/
function GLS_getAuthors()
{
    global $_TABLES;
    
    $authors = array();
    $result = DB_query("SELECT DISTINCT (owner) FROM {$_TABLES['gls_entry']}");
    $nrows  = DB_numRows($result);
    for ($i = 1; $i<=$nrows; $i++) {
        $bArray = DB_fetchArray($result);
        $authors[] = $bArray['owner'];
    }
    $result = DB_query("SELECT DISTINCT (last_owner) FROM {$_TABLES['gls_entry']}");
    $nrows  = DB_numRows($result);
    for ($i = 1; $i<=$nrows; $i++) {
        $bArray = DB_fetchArray($result);
        $authors[] = $bArray['last_owner'];
    }
    return array_unique($authors);
}

/**
 * Returns a list of all characters for the menu.
 * Used characters in glossary entries are marked with their occurences.
 **/ 
function GLS_getMenuCharacters($pCategory, $pProject, $pTimeLimit="")
{
    global $_GLS_CHARACTERS, $_TABLES;
    
    $lCharacters = array();
    foreach (str_split($_GLS_CHARACTERS) as $bChar) {
        $lCharacters[$bChar] = 0;
    }
    
    // Assemble WHERE-clause
    $selection = ['category' => $pCategory, 'project' => $pProject, 'timelimit' => $pTimeLimit];
    $lWhereClause = GLS_assembleWhereClause($selection);

    // Read all used letters for given category and project
    $result = DB_query("SELECT DISTINCT letter, count(*) as summe from {$_TABLES['gls_entry']} $lWhereClause GROUP BY letter");
    $nrows  = DB_numRows($result);
    for ($i = 1; $i<=$nrows; $i++) {
        $bArray = DB_fetchArray($result);
        $lCharacters[$bArray['letter']] = $bArray['summe'];
    }

    return $lCharacters;
}

/**
 * Returns a list of all abbrevations for the menu.
 * @TODO limit query to 11 and skip current abbreviation
 **/ 
function GLS_getMenuAbbreviations($pCategory,$pProject,$pCharacter, $pTimeLimit="")
{
    global $_TABLES, $_GLS_CONF;

    $lAbbreviations = array();
    
    // Assemble WHERE-clause
    $selection = ['category' => $pCategory, 'project' => $pProject, 'letter' => $pCharacter, 'timelimit' => $pTimeLimit];
    $lWhereClause = GLS_assembleWhereClause($selection);

    // Read all used letters for given category and project
    $result = DB_query("SELECT DISTINCT abbreviation from {$_TABLES['gls_entry']} $lWhereClause ORDER BY abbreviation LIMIT {$_GLS_CONF['limitterms']}");
    $nrows  = DB_numRows($result);
    for ($i = 1; $i<=$nrows; $i++) {
        $bArray = DB_fetchArray($result);
        $lAbbreviations[] = $bArray['abbreviation'];
    }

    return $lAbbreviations;
}

/**
 * Returns a HTML listbox with 'added during' options
 **/
function GLS_getAddedDuringOptions($pAddedDuring)
{
    global $LANG_GLS_00;
    
    $lReturn = '<select onchange="this.form.submit()" style="border: 1px solid gray; background-color: white;" name="addedDuring" size="1">';
    $lOptions = array ( '',
                        $LANG_GLS_00['last_hour'], 
                        $LANG_GLS_00['today'], 
                        $LANG_GLS_00['last_week'], 
                        $LANG_GLS_00['last_month'], 
                        $LANG_GLS_00['last_year']);

    foreach ( $lOptions as $bOption ) {
        if ( strcmp($bOption, $pAddedDuring) == 0 ) {
            $lReturn .= '<option selected=selected style="color: blue;">'. $bOption .'</option>';
        } else {
            $lReturn .= '<option>'. $bOption .'</option>';
        }
    }

    $lReturn .= '</select>';
    return $lReturn;
}

function GLS_getColumnNames($tableName)
{
    $columns = array();
    $sql = "SHOW COLUMNS FROM $tableName ";
    $result = DB_query($sql);
    $nrows  = DB_numRows($result);
    for ($i = 1; $i<=$nrows; $i++) {
        $bArray = DB_fetchArray($result);
        $columns[] = $bArray['Field'];
    }
    return $columns;
}

/**
 * Prepares explanation string for save.
 * This includes converting used abbreviations into HTML-links
 **/
function GLS_insertGlossaryLinks ( $pExplanation, $pAbbreviation = '', $pProject = 0 ) 
{
    global $_TABLES, $_CONF, $_GLS_CONF;
    
    // Change strings 
//  $lExplanation = addslashes(htmlspecialchars($pExplanation));
    $lExplanation = strip_tags($pExplanation);

    $matches = [];
    $tags    = [];
    if (preg_match_all('/\b([\w\-]+)\b/', $lExplanation, $matches)) {
        foreach ($matches[1] as $tag) {
            if (GLS_isValidAbbreviation($tag)){
               $tags[] = strtolower($tag);
            }
        }
        $tags = array_unique($tags);
    }

    $SQL = "SELECT abbreviation, explanation, categories.value AS 'category_name', projects.value AS 'project_name' "
         . "FROM ({$_TABLES['gls_entry']} AS entries) "
         . "LEFT JOIN {$_TABLES['gls_category']} AS categories ON (categories.id = entries.category) "
         . "LEFT JOIN {$_TABLES['gls_project']}  AS projects   ON (projects.id   = entries.project) "
         . "WHERE LCASE(abbreviation) != LCASE('$pAbbreviation') "
         . "AND (entries.project = $pProject OR $pProject = 0) "
         . "AND ( LCASE(abbreviation) IN ('" .join("','", $tags). "') OR abbreviation LIKE '% %') "
         . "ORDER BY CHAR_LENGTH(abbreviation) DESC, abbreviation, project, category";
    $result = DB_query($SQL);
    $tags   = [];
    // Extract related abbreviations and link them!
    $nrows  = DB_numRows($result);
    for ($i = 1; $i<=$nrows; $i++) {
        $bEntry = DB_fetchArray($result);
        $tExplanation = GLS_truncate($bEntry['explanation']);
        // cumulate advisory content in array
        $tag = strtolower($bEntry['abbreviation']);
        $tags[$tag] .= "[{$bEntry['project_name']}/{$bEntry['category_name']}] {$tExplanation}";
    }
    // make sure the longer (whitespaced) term is coming first
    $lExplanation = $pExplanation;
    foreach ($tags as $tag => $content) {
        // make sure complete replacements are skipped
        $bMatchString = "/<span .*?<\/span>(*SKIP)(*FAIL)|\b({$tag})\b/is";
        // Replace word(s) with link around it
        $bReplacement = '<span title="'.$content.'">'
                      . '<a href="'.$_CONF['site_url'].'/glossary/index.php?mode=view&abbreviation='.$tag.'">'
                      . '\1</a></span>';
        $lExplanation = preg_replace($bMatchString, $bReplacement, $lExplanation, 1 );
    }
    return $lExplanation;
}

function GLS_errorLog($message, $flag = 0)
{
    global $_CONF;
    $logfile   = $_CONF['path_log'] . 'glossary.log';
    $timestamp = @strftime('%c');

    if ($file = fopen($logfile, 'a')) {
        fputs($file, "$timestamp - $message \n");
        fclose($file);
    }
    if ($flag == 2) {
        return $message;
    }
}

function GLS_truncate($text)
{
    global $_GLS_CONF;
    $tExplanation = strip_tags($text);
    // compute the length of the advisory content
    $length = strpos($tExplanation, "\n") - 1;
    if ($length > 0) {
        $length = min($length, $_GLS_CONF['max_characters']);
    } else {
        $length = $_GLS_CONF['max_characters'];
    }
    if (strlen($tExplanation) > $length && $length > 0) {
        $tExplanation = substr($tExplanation, 0, $length) . ' ...';
    }
    return $tExplanation."\n";
}
function GLS_render($content, $layout = '', $headers = '')
{
    global $_GLS_CONF;
    $information = array();
    $information['rightblock'] = $_GLS_CONF['right_blocks'];
    switch ($layout) {
        case 'menu' :
            $information['what'] = 'menu';
            break;
        case 'all' :
            $information['what'] = 'menu';
            $information['rightblock'] = true;
            break;
        default     :
            break;
    }
    if (!empty($headers)) {
        $information['headercode'] = $headers;
    }
//  $information['pagetitle']  = 
    if (COM_versionCompare(VERSION, "2.0.0",  '>=')) {
        return COM_createHTMLDocument($content, $information);
    } else {
        if (!empty($headers)) {
            $display .= "<html><head>\n";
            $display .= $headers;
            $display .= "</head>\n";
        }
        return COM_siteHeader($information['what'], $information['pagetitle'])
             . $display
             . COM_siteFooter($information['rightblock']);
    }
}
?>