<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | Glossary Plugin                                                           |
// +---------------------------------------------------------------------------+
// | entry.php                                                                 |
// | Glossary entry view/edit/delete page                                      |
// |                                                                           |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2003 by the following authors:                              |
// | Alexander Schmacks          -    alexander@schmacks.de                    |
// | Copyright (C) 2021 by the following authors:                              |
// | Remy Kobolski               -    remy@nlbox.com                           |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+
//
// $Id: entry.php,v 1.2 2004/01/22 07:46:48 schmacks Exp $


/**
* Geeklog commong function library
*/
include('../lib-common.php');

if (!in_array('glossary', $_PLUGINS)) {
    COM_handle404();
    exit;
}

foreach (['mode', 'id', 'page', 'letter', 'abbreviation', 'category', 'project', 'msg'] as $keyword) {
    if (isset ($_GET[$keyword])) {
        $$keyword = COM_applyFilter($_GET[$keyword]);
    }
}
foreach (['mode', 'id', 'owner', 'timestamp', 'abbreviation', 'category', 'project'] as $keyword) {
    if (isset ($_POST[$keyword])) {
        $$keyword = COM_applyFilter($_POST[$keyword]);
    }
}
$explanation = $_POST['explanation'];  // Filtering in entryEditor

// Check security depending on the different modes
if (   ( $mode=='view' && !($_GLS_CONF['show_to_anyone'] || SEC_hasRights('glossary.view')) ) 
    || ( ($mode=='edit' || $mode=='submit') && !SEC_hasRights('glossary.edit') )
    || ( $mode==$LANG24[11] && !SEC_hasRights('glossary.admin') ) 
    ) {
    // Someone is trying to illegally access this page
    COM_errorLog("Someone has tried to illegally access the Glossary page.  User id: {$_USER['uid']}, Username: {$_USER['username']}, IP: $REMOTE_ADDR",1);
    $display .= COM_startBlock($LANG_GLS_00['access_denied']);
    $display .= $LANG_GLS_00['access_denied_msg'];
    $display .= COM_endBlock();
    $display  = GLS_render($display, 'all');
    COM_output($display);
    exit;
}

$display = '';

/**
 * Previews an glossary entry
 *
 * @param   Array   $pEntry     Array containing the glossary entry elements
 * @param   String  $pMode      Mode in which the entry is displayed (e.g. 'view' or 'edit' etc.)
 **/
function previewEntry($pEntry, $pMode)
{
    global $_CONF, $LANG_GLS_00, $LANG24, $_TABLES, $_SERVER;
    
    // Get list of categories and of projects
    $categories = GLS_getCategories();
    $projects   = GLS_getProjects();

    // Output each entry with template
    $T = new Template($_CONF['path'] . 'plugins/glossary/templates');
    $T->set_file('entry','entry_detail.thtml');

    // Abbreviation & Explanation
    $T->set_var('abbreviation', $pEntry['abbreviation'] );

    if (!GLS_isValidAbbreviation($pEntry['abbreviation'])) {
        $T->set_var('validation_result', '<b> INVALID </b>');
    }
    $lExplanation = nl2br(preg_replace('/\>\n/', '>', $pEntry['explanation']));
    $lExplanation = GLS_insertGlossaryLinks($lExplanation, $pEntry['abbreviation'], $pEntry['project']);
    $T->set_var('explanation', $lExplanation);

    // View link (to detail/solo view)
    $T->set_var('view_link_label', $LANG_GLS_00['detail_view']);
    $T->set_var('view_link', $_CONF['site_url'] .'/glossary/entry.php?mode=view&id='. $pEntry['id']);
    
    // Category & Project
    $T->set_var('category_label', $LANG_GLS_00['category']);
    $T->set_var('category', '<a href="'.$_CONF['site_url'] .'/glossary/index.php?category='. $pEntry['category'] .'">'. $categories[$pEntry['category']] .'</a>');
    $T->set_var('project_label', $LANG_GLS_00['project']);
    $T->set_var('project', '<a href="'.$_CONF['site_url'] .'/glossary/index.php?project='. $pEntry['project'] .'">'. $projects[$pEntry['project']] .'</a>');
    
    // Owner info
    $T->set_var('owner_label', $LANG_GLS_00['owned_by']);
    $lUsername = DB_getItem( $_TABLES['users'],'username',"uid = '{$pEntry['owner']}'" );
    $T->set_var('owner', '<a href="'. $_CONF['site_url'] .'/users.php?mode=profile&uid='.$pEntry['owner'].'">'.$lUsername.'</a>');

    if ( !empty($pEntry['last_owner']) && $pEntry['last_owner']!=$pEntry['owner'] ) {
        $lUsername = DB_getItem( $_TABLES['users'],'username',"uid = '{$pEntry['last_owner']}'" );
        $T->set_var('last_owner_label', $LANG_GLS_00['last_owner']);
        $T->set_var('last_owner', '<a href="'. $_CONF['site_url'] .'/users.php?mode=profile&uid='.$pEntry['last_owner'].'">'.$lUsername.'</a>');
    }
    
    // Timestamp
    $bCurTime = COM_getUserDateTimeFormat($pEntry['timestamp']);
    $T->set_var('timestamp_label', $LANG_GLS_00['creation_on'] );
    $T->set_var('timestamp', $bCurTime[0] );
    if ( isset($pEntry['last_timestamp']) && $pEntry['last_timestamp']!=$pEntry['timestamp'] ) {
        $bCurTime = COM_getUserDateTimeFormat($pEntry['last_timestamp']);
        $T->set_var('last_timestamp_label', $LANG_GLS_00['last_edit_on'] );
        $T->set_var('last_timestamp', $bCurTime[0] );
    }

    // Actions only when in view mode
    if ( $pMode == 'view' ) {
        if ( SEC_hasRights('glossary.edit,glossary.admin','OR') ) {
            $T->set_var('edit_entry', '<a href="entry.php?mode=edit&id='.$pEntry['id'].'">'. $LANG_GLS_00['edit_entry'] .'</a>');
        }
        if ( SEC_hasRights('glossary.admin') ) {
            $T->set_var('delete_entry_linkstart', '<a href="entry.php?mode=' . $LANG24[11] . '&id=' . $pEntry['id'] . '" '
                . 'onclick="return confirm(\'' . $LANG_GLS_00['deletion_inquiry'] . '\');">');
            $T->set_var('delete_entry', $LANG24[11]);
            $T->set_var('delete_entry_linkend', '</a>');
        }
    }

    // If in view mode, present a backward naviation
    $lBackNavigation = $LANG_GLS_00['back_to_index'] 
                       . ' <img src="'.$_CONF['site_url'].'/glossary/images/Link_Back.png" border="0" title="'
                       . $LANG_GLS_00['back_to_index']   .'" style="vertical-align: text-top;"></a>';
    if ( $pMode == 'view' ) {
        $lBackNavigation = '<a href="'.$_CONF['site_url'] .'/glossary/index.php">'. $lBackNavigation;
        $T->set_var('back_navigation', '<br>'.$lBackNavigation);
    } elseif ( $pMode != 'edit'  && $pMode != $LANG24[9] )  {
        // If in other mode than view, edit or preview, refresh the page
        $lBackNavigation = '<a href="'.$_SERVER['HTTP_REFERER'] .'">'. $lBackNavigation;
        $T->set_var('back_navigation', '<br>'.$LANG_GLS_00['page_refresh'].'<br>'.$lBackNavigation);
    }
    
    $T->parse('output','entry');
    $retval = $T->finish($T->get_var('output'));
    
    return $retval;
}

/**
* Shows entry editor
*
* Displays the story entry form
*
* @param    string      $sid    ID of entry to edit
* @param    string      $mode   ??
* @return   string      HTML for story editor
*
*/
function entryEditor($pID = '', $pMode = '', $preView = true) 
{
    global $_TABLES, $_USER, $_CONF, $LANG24, $LANG_GLS_00, $_GLS_CONF, $LANG12, $LANG_ACCESS,
            $id, $owner, $abbreviation, $project, $category, $explanation;     // Given values by HTML parameter

    // Get entry information depending on actual mode
    if (($pMode == 'edit' || $pMode=='view' || $pMode=='view_after') && (isset($pID) && $pID!=0)) {
        // Edit mode
        $result = DB_query("SELECT * FROM {$_TABLES['gls_entry']} WHERE id = '$pID'");
        $A = DB_fetchArray($result);
    } else {
        $A = compact('id', 'owner', 'abbreviation', 'category', 'project');
        if ($pMode == 'submit') {
            // Submit mode
            $A['owner']     = $_USER['uid'];
            $A['timestamp'] = date("Y-m-d H:i:s",time());
        } else {
            // View mode
            $A['explanation'] = $explanation;
            if (empty($owner)) {
                $A['owner'] = $_USER['uid'];
            }
        }
    }

    // Load HTML templates
    $story_templates = new Template($_CONF['path'] . 'plugins/glossary/templates');
    $story_templates->set_file(array('editor'=>'editor.thtml'));

    $story_templates->set_var('site_url', $_CONF['site_url']);
    $story_templates->set_var('site_admin_url', $_CONF['site_admin_url']);
    $story_templates->set_var('layout_url', $_CONF['layout_url']);

    // Default timestamp
    if (empty($A['timestamp'])) {
        $A['timestamp'] = date("Y-m-d H:i:s",time());
    }
    
    // Show preview if we already have some values (would be any mode but 'submit')
    if (!empty($A['explanation']) || $mode==$LANG24[9]) {
        if ($preView) {
            // Only show block when not in view mode
            if ($pMode != 'view') {
                $display .= COM_startBlock($LANG12[32]);
            }
            $display .= previewEntry($A, $pMode);
            if ($pMode != 'view') {
                $display .= COM_endBlock();
            }
        }
    }

    // In view mode we return from this method right here
    if ($pMode == 'view'  ||  $pMode == 'view_after') {
        return $display;
    }

    $display .= COM_startBlock($LANG_GLS_00['entry_editor']);

    // Prepare text for output into edit area:
    // - Replace <br> with linefeed
    $A['explanation'] = preg_replace( "/<br>/", "\n", $A['explanation'] );
    // - Delete HTML-Tags
    $A['explanation'] = strip_tags($A['explanation'], $_GLS_CONF['allowed_tags']);

    // Owner info
    $story_templates->set_var('owner_label', $LANG_GLS_00['owner']);
    $story_templates->set_var('owner', DB_getItem($_TABLES['users'],'username',"uid = {$A['owner']}"));
    $story_templates->set_var('owner_id', $A['owner']);
    $story_templates->set_var('last_owner_id', $A['last_owner']);
    
    if (!empty($owners)) {
        if ( SEC_hasRights('glossary.admin') ) {
            // Get all users for holding usernames
            $owners = GLS_getAuthors();
            $users  = array();
            $lSQL = "SELECT uid, username FROM {$_TABLES['users']} WHERE uid IN (".join(',',$owners).")";
            $result = DB_query($lSQL);
            $nrows  = DB_numRows($result);
            for ($i = 1; $i<=$nrows; $i++) {
                $bArray = DB_fetchArray($result);
                $users[$bArray['uid']] = $bArray['username'];
            }
            $story_templates->set_var('owner', '<select style="border: 1px solid gray; background-color: white;" name="newOwner">');
            $story_templates->set_var('owner', " \n", true);
            $story_templates->set_var('owner', "<option value='0'> </option> \n", true);
            foreach ($users as $uid => $username) {
                if ($uid == $A['owner']) {
                    $selected = 'selected=selected ';
                } else{
                    $selected = '';
                }
                $story_templates->set_var('owner', "<option value='{$uid}' $selected>{$username}</option> \n", true);
            }
            $story_templates->set_var('owner', "</select> \n", true);
        }
    }

    // Hidden ID field
    $story_templates->set_var('entry_id', $A['id']);

    // Timestamp
    $bCurTime = COM_getUserDateTimeFormat($A['timestamp']);
    $story_templates->set_var('timestamp_label', $LANG_GLS_00['creation_on']);
    $story_templates->set_var('timestamp', $bCurTime[0]);
    // Keep real & last timestamp for storage purpose. The one above is just for display
    $story_templates->set_var('tech_timestamp', $A['timestamp']);
    $story_templates->set_var('last_timestamp', $A['last_timestamp']);

    // Category & Project
    $story_templates->set_var('category_label', $LANG_GLS_00['category']);
    $story_templates->set_var('category_options', COM_optionList($_TABLES['gls_category'],'id, value',$A['category'], 0));
    $story_templates->set_var('project_label', $LANG_GLS_00['project']);
    $story_templates->set_var('project_options',  COM_optionList($_TABLES['gls_project'], 'id, value',$A['project'], 0));
    
    // Abbreviation & Explanation
    $story_templates->set_var('abbreviation_label', $LANG_GLS_00['abbreviation']);
    $A['abbreviation'] = str_replace('$','&#36;',$A['abbreviation']);
    $A['abbreviation'] = str_replace('{','&#123;',$A['abbreviation']);
    $A['abbreviation'] = str_replace('}','&#125;',$A['abbreviation']);
    $story_templates->set_var('abbreviation', stripslashes($A['abbreviation']));
    
    $story_templates->set_var('explanation_label', $LANG_GLS_00['explanation']);
    $A['explanation'] = htmlspecialchars(COM_checkWords(stripslashes($A['explanation'])));
    $story_templates->set_var('explanation', $A['explanation']);
    
    $story_templates->set_var('explanation_help', $LANG_GLS_00['explanation_help']);
    if (!empty($_GLS_CONF['allowed_tags'])) {
        $story_templates->set_var('allowed_tags', $LANG_GLS_00['allowed_tags'] . htmlentities(join(',', $_GLS_CONF['allowed_tags'])));
    }
    
    // Actions
    if (SEC_hasRights('glossary.admin')) {
        $story_templates->set_var('delete_option', '<input type="submit" value="' . $LANG24[11] . '" name="mode" '
            . 'onclick="return confirm(\'' . $LANG_GLS_00['deletion_inquiry'] . '\');">');
    }
    $story_templates->set_var('story_id', $A['sid']);
    $story_templates->set_var('save_label', $LANG24[8]); 
    $story_templates->set_var('preview_label', $LANG24[9]); 
    $story_templates->set_var('cancel_label', $LANG24[10]); 


    // Parse and output template
    $story_templates->parse('output','editor');
    $display .= $story_templates->finish($story_templates->get_var('output'));
    $display .= COM_endBlock();

    return $display;
}

/** 
 * Saves entry to database
 **/
function submitEntry($pID,$pOwner,$pTimestamp,$pAbbreviation,$pCategory,$pProject,$pExplanation) 
{
    global $_USER, $_TABLES, $_CONF, $_GLS_CONF, $LANG_GLS_00;
        
    // If main fields are given, we can save the entry
    if (!empty($pAbbreviation) && !empty($pExplanation)) {

        // If timestamp is empty, fill it (should not happen, but who knows)        
        if (empty($pTimestamp)) {
            $pTimestamp = date("Y-m-d H:i:s",time());
        }
        
        // Save letter for glossary navigation
        $lLetter = strtoupper(substr(trim($pAbbreviation),0,1));
        if (is_numeric($lLetter)) {
            $lLetter = '#';
        }

        // No ID yet? Then we have to get one!
        if ( empty($pID) || $pID == 0 ) {
            if (DB_getItem($_TABLES['gls_entry'],'id',"LCASE(abbreviation) = LCASE('{$pAbbreviation}') AND project = $pProject AND category = $pCategory")) {
                $display .= "<h2>Term '{$pAbbreviation}' is a duplicate</h2>";
                $display .= entryEditor($pID, 'retry', false);
                $display = GLS_render($display, 'menu');
                COM_output($display);
                exit;
            } else {
                $pID = 0;
                // $pID = GLS_getNewEntryID();
                // Last owner and timestamp are not different at first
                $lLastTimestamp = $pTimestamp;
                $lLastOwner = $pOwner;
            }
        } else {
            // Save information of last edit, when entry was edited (not created)
            $lLastTimestamp = date("Y-m-d H:i:s",time());
            $lLastOwner = $_USER['uid'];
        }

        // Change strings 
        if (!GLS_isValidAbbreviation($pAbbreviation)) {
            $display .= "<h2>Term '{$pAbbreviation}' is invalid</h2>";
            $display .= entryEditor($pID, 'retry', false);
            $display = GLS_render($display, 'menu');
            COM_output($display);
            exit;
        } else {

            $pAbbreviation = addslashes($pAbbreviation);
            $pExplanation  = addslashes(strip_tags($pExplanation, $_GLS_CONF['allowed_tags']));

            // Finally save the whole stuff... ;)
            DB_save($_TABLES['gls_entry'],
                    'id,letter,abbreviation,explanation,category,project,owner,timestamp,last_owner,last_timestamp',
                    "$pID,'$lLetter','$pAbbreviation','$pExplanation',$pCategory,$pProject,$pOwner,'$pTimestamp',$lLastOwner,'$lLastTimestamp'");
            if (empty($pID)) {
               $pID = DB_insertId();
            }
            if (function_exists('COM_redirect')) {
                COM_redirect($_CONF['site_url'] . '/glossary/entry.php?msg=9&mode=view_after&id=' . $pID);
            } else {
                $display = COM_refresh($_CONF['site_url'] . '/glossary/entry.php?msg=9&mode=view_after&id=' . $pID);
                COM_output($display);
                exit;
            }
        }
    } else {
        // Message about missing fields         
        $display .= GLS_errorLog($LANG_GLS_00['missing_fields'],2);
        $display .= entryEditor($pID);
        $display  = GLS_render($display, 'menu');
        COM_output($display);
        exit;
    }
}



// MAIN ---------------------------------------------------------------------------
    
$display = '';

switch ($mode) {
    case $LANG24[11] :    // DELETE the entry
        if (!isset ($id) || empty ($id) || ($id == 0)) {
            GLS_errorLog ('Attempted to delete entry id=' . $id);
            $display .= COM_refresh ($_CONF['site_url'] . '/glossary/index.php');
        } else {
            DB_delete(  $_TABLES['gls_entry'], 'id', $id, $_CONF['site_url'] . '/glossary/entry.php?msg=10');
            // deleteEntry redirects the page. Nothing to do here anymore!
        }
        break;
    case 'submit'   :
    case $LANG24[9] :    // PREVIEW the entry
        $display .= entryEditor($id,$mode);
        $display  = GLS_render($display, 'menu');
        COM_output($display);
        break;
    case 'view'     :    // VIEW the entry
        if ( isset($msg) ) {
            $display .= COM_showMessage($msg);
        }
        $display .= entryEditor($id,'view');
        $display  = GLS_render($display, 'menu');
        COM_output($display);
        break;
    case 'edit'     :    // EDIT the entry
        $display .= entryEditor($id,$mode,false);
        $display = GLS_render($display, 'menu');
        COM_output($display);
        break;
    case $LANG24[8] :    // SAVE the entry
        submitEntry($id,$owner,$timestamp,$abbreviation,$category,$project,$explanation);
        // submitEntry redirects the page. Nothing to do here anymore!
        break;
    default         :    // No mode. Just show message and entry. Refresh to glossary main page.
        $headers = "<meta http-equiv='Refresh' content='". $_GLS_CONF['refresh_time'] ."; url=". $_CONF['site_url'] ."/glossary/index.php' />\n";
        // If message is given, show it
        if ( isset($msg) ) {
            $display .= COM_showMessage($msg);
        }
        $display .= entryEditor($id,'view_after');
        if ( $mode != $LANG24[10] && $mode != 'view_after' ) {
            $display .= '<br><br>'.$LANG_GLS_00['page_refresh'].' <a href="'.$_CONF['site_url'] .'/glossary/index.php">'. $LANG_GLS_00['back_to_index'] .'</a>';
        }
        $display = GLS_render($display, 'menu', $headers);
        COM_output($display);
        break;

}

?>