<?php

// +---------------------------------------------------------------------------+
// | Glossary Plugin for Geeklog - The Ultimate Weblog                         |
// +---------------------------------------------------------------------------+
// | index.php                                                                 |
// |                                                                           |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2003/4 by the following authors:                            |
// | Alexander Schmacks          -    alexander@schmacks.de                    |
// | Copyright (C) 2021 by the following authors:                              |
// | Remy Kobolski               -    remy@nlbox.com                           |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+
//

require_once('../lib-common.php');

if (!in_array('glossary', $_PLUGINS)) {
    COM_handle404();
    exit;
}
if ( !$_GLS_CONF['show_to_anyone']  &&  !SEC_hasRights('glossary.view')) {
    // Someone is trying to illegally access this page
    COM_errorLog("Someone has tried to illegally access the Glossary page.  User id: {$_USER['uid']}, Username: {$_USER['username']}, IP: $REMOTE_ADDR",1);
    $display .= COM_startBlock($LANG_GLS_00['access_denied']);
    $display .= $LANG_GLS_00['access_denied_msg'];
    $display .= COM_endBlock();
    $display  = GLS_render($display);
    COM_output($display);
    exit;
}

foreach (['mode', 'id', 'page', 'letter', 'abbreviation', 'category', 'project', 'msg'] as $keyword) {
    if (isset ($_GET[$keyword])) {
        $$keyword = COM_applyFilter($_GET[$keyword]);
        $selection[$keyword] = $$keyword;
    }
}
foreach (['mode', 'id', 'owner', 'timestamp', 'abbreviation', 'category', 'project'] as $keyword) {
    if (isset ($_POST[$keyword])) {
        $$keyword = COM_applyFilter($_POST[$keyword]);
        $selection[$keyword] = $$keyword;
    }
}
$explanation = $_POST['explanation'];  // Filtering in entryEditor

// Page handling for a bigger number of entries
if (empty($page)) {
    // If no page sent then assume the first.
    $page = 1;
}

/** 
 * Main Function
 **/
$T = new Template($_CONF['path'] . 'plugins/glossary/templates');
$T->set_file('page','index.thtml');
$T->set_var('header', $_CONF['site_name'] .' '. $LANG_GLS_00['shortname']);
$T->set_var('site_url',$_CONF['site_url']);

// Assemble QueryString of next URL

$queryString = GLS_assembleUrlParameters($selection);
// Give admins and editors right to submit new entries
if ( SEC_hasRights('glossary.edit,glossary.admin','OR')) {
    $lURL = 'entry.php'. $queryString;
    $T->set_var('submit_entry', '<a href="'. $lURL .'mode=submit">'. $LANG_GLS_00['submit_entry'] .'</a>');
    $T->set_var('glossary_moderator',  $_USER['username']);
}

foreach (['addedDuring', 'contributed'] as $keyword) {
    if (isset ($_REQUEST[$keyword])) {
        $$keyword = COM_applyFilter($_REQUEST[$keyword]);
        $selection[$keyword] = $$keyword;
    }
}

// ------------------------------------------------------------------
// Add search form for latest additions
// Give admins and editors right to filter the entries
if ( SEC_hasRights('glossary.edit,glossary.admin','OR')) {
    $lURL = 'index.php?'.$_SERVER['QUERY_STRING'];
    $additionsForm  = '<form action="' .$lURL. '" method="post" style="display: inline;" >'. $LANG_GLS_00['added_during'] .' ';
    $additionsForm .= GLS_getAddedDuringOptions($addedDuring);
    $additionsForm .= '</form>';
    $T->set_var('added_during_form', $additionsForm);
}

// Prepare search limit by timestamp, according to given selection from the form above
if ( !empty($addedDuring) ) {
    $lCompareTimestamp = '';
    switch ( $addedDuring ) {
        case $LANG_GLS_00['last_hour']:
            $lCompareTimestamp = strftime("%Y-%m-%d %H:%M:%S", time() - 3600);
            break;
        case $LANG_GLS_00['today']:
            $lCompareTimestamp = strftime("%Y-%m-%d 00:00:01", time());
            break;
        case $LANG_GLS_00['last_week']:
            $lCompareTimestamp = strftime("%Y-%m-%d %H:%M:%S", time() - 604800);
            break;
        case $LANG_GLS_00['last_month']:
            $lCompareTimestamp = strftime("%Y-%m-%d %H:%M:%S", time() - 2592000);
            break;
        case $LANG_GLS_00['last_year']:
            $lCompareTimestamp = strftime("%Y-%m-%d %H:%M:%S", time() - 31536000);
            break;
        default:
            $lTimeLimit = '';
            break;
    }
    if ( !empty($lCompareTimestamp) ) {
        $lTimeLimit = "last_timestamp > '$lCompareTimestamp'";
        $selection['timelimit'] = $lTimeLimit;
    }
}
// ------------------------------------------------------------------
// Add search form for users that contributed
// Get all users for holding usernames
$owners = GLS_getAuthors();
$users  = array();
if (!empty($owners)) {
    $lSQL = "SELECT uid, username FROM {$_TABLES['users']} WHERE uid IN (".join(',',$owners).")";
    $result = DB_query($lSQL);
    $nrows  = DB_numRows($result);
    for ($i = 1; $i<=$nrows; $i++) {
        $bArray = DB_fetchArray($result);
        $users[$bArray['uid']] = $bArray['username'];
    }
    // Give admins and editors right to filter the entries
    if ( SEC_hasRights('glossary.edit,glossary.admin','OR') ) {
        $lURL = 'index.php?'.$_SERVER['QUERY_STRING'];
        $T->set_var('contributed_label',   'Contributed by');
        $T->set_var('glossary_moderator',  $_USER['username']);
        $T->set_var('contributed_options', '<form action="' .$lURL. '" method="post" style="display: inline;" >');
        $T->set_var('contributed_options', '<select onchange="this.form.submit()" style="border: 1px solid gray; background-color: white;" name="contributed">', true);
        $T->set_var('contributed_options', " \n", true);
        $T->set_var('contributed_options', "<option value='0'> </option> \n", true);
        foreach ($users as $uid => $username) {
            if ($uid == $contributed) {
                $selected = 'selected=selected ';
            } else{
                $selected = '';
            }
            $T->set_var('contributed_options', "<option value='{$uid}' $selected>{$username}</option> \n", true);
        }
        $T->set_var('contributed_options', "</select></form> \n", true);
    }
}

// ------------------------------------------------------------------
// Add categories to template
$myQueryString = $selection;
unset($myQueryString['category']);
unset($myQueryString['abbreviation']);
$selfURL = 'index.php' . GLS_assembleUrlParameters($myQueryString);

if ( isset($category) ) {
    $categoryMenu = '<a href="'.$selfURL.'">'.$LANG_GLS_00['all'].'</a> ';
} else {
    $categoryMenu = '<u>'.$LANG_GLS_00['all'].'</u> &nbsp; ';
}

$categories = GLS_getCategories($selection);

foreach ($categories as $bID => $bText) {
    if ( $bID != $category ) {
        $categoryMenu .= '<b>&middot;</b> &nbsp; <a href="'.$selfURL.'category='.$bID.'">'.$bText.'</a> &nbsp; ';
    } else {
        $categoryMenu .= '<b>&middot;</b> &nbsp; <u>'.$bText.'</u> &nbsp; ';
    }
}
$T->set_var('categories_label',$LANG_GLS_00['categories']);
$T->set_var('categories',$categoryMenu);


// ------------------------------------------------------------------
// Add projects to template
$myQueryString = $selection;
unset($myQueryString['category']);
unset($myQueryString['project']);
unset($myQueryString['abbreviation']);
$selfURL = 'index.php' . GLS_assembleUrlParameters($myQueryString);

if ( isset($project) ) {
    $projectMenu = '<a href="'.$selfURL.'">'.$LANG_GLS_00['all'].'</a> ';
} else {
    $projectMenu = '<u>'.$LANG_GLS_00['all'].'</u> &nbsp; ';
}

$projects = GLS_getProjects($selection);

foreach ($projects as $bID => $bText) {
    if ( $bID != $project ) {
        $projectMenu .= '<b>&middot;</b> &nbsp; <a href="'.$selfURL.'project='.$bID.'">'.$bText.'</a> &nbsp; ';
    } else {
        $projectMenu .= '<b>&middot;</b> &nbsp; <u>'.$bText.'</u> &nbsp; ';
    }
}
$T->set_var('projects_label',$LANG_GLS_00['projects']);
$T->set_var('projects',$projectMenu);


// ------------------------------------------------------------------
// Now show menu for letter selection
$myQueryString = $selection;
unset($myQueryString['letter']);
unset($myQueryString['abbreviation']);
$selfURL = 'index.php' . GLS_assembleUrlParameters($myQueryString);

if ( isset($letter) ) {
    $lettersMenu = '<a href="' .$selfURL. '">' .$LANG_GLS_00['all']. '</a> ';
} else {
    $lettersMenu = '<u>'.$LANG_GLS_00['all'].'</u> ';
}

$letters = GLS_getMenuCharacters($category,$project,$lTimeLimit);

foreach ($letters as $bChar => $bCount) {
    $lTitle = "title=\"{$bCount} entries\"";
    if ( $bChar == $letter ) {
        $lettersMenu .= ' &nbsp;<span style="border:2px solid green; border-radius: 8px; padding: 3px;" '.$lTitle.'><b><u>'.$bChar.'</u></b></span>';
    } else {
        if ( $bCount == 0 ) {
            $lettersMenu .= ' &nbsp;<span style="color: #999999">'.$bChar.'</span>';
        } else {
            $lettersMenu .= ' &nbsp;<a href="'.$selfURL.'letter='.$bChar.'" '.$lTitle.'><b>'.$bChar.'</b></a> ';
        }
    }
}
$T->set_var('characters_label', $LANG_GLS_00['characters']);
$T->set_var('characters', $lettersMenu);


// ------------------------------------------------------------------
// Now show menu for abbreviation selection
$myQueryString = $selection;
unset($myQueryString['abbreviation']);
$selfURL = 'index.php' . GLS_assembleUrlParameters($myQueryString);

$abbreviations = GLS_getMenuAbbreviations($category,$project,$letter,$lTimeLimit);

if ( sizeof($abbreviations)== 0 ) {
    $abbreviationMenu .= '<i>'. $LANG_GLS_00['no_entries'] .'</i>';
} else {
    for ($i = 0; $i < $_GLS_CONF['limitterms']; $i++) {
        $bAbbrev = $abbreviations[$i];
        if ( $bAbbrev != $abbreviation ) {
            $abbreviationMenu .= ' &nbsp;<a href="'.$selfURL.'abbreviation='.$bAbbrev.'">'.$bAbbrev.'</a>&nbsp; ';
        } else {
            $abbreviationMenu .= ' &nbsp;<u>'.$bAbbrev.'</u>&nbsp; ';
        }
    }
}
if ( sizeof($abbreviations) > $_GLS_CONF['limitterms'] - 1 ) {
    $abbreviationMenu .= '<i>..more</i>';
}
$T->set_var('abbreviations_label', $LANG_GLS_00['abbreviations']);
$T->set_var('abbreviations',$abbreviationMenu);

$T->parse('output','page');
$display .= $T->finish($T->get_var('output'));

// ------------------------------------------------------------------
// Show the glossary entries
$tWhereClause = GLS_assembleWhereClause($selection);
// Select glossary entries to show (with the given WHERE clause)
$limit = $_GLS_CONF['limitentries'];
$offset = ($page - 1) * $limit;
$lSQL = "SELECT * from {$_TABLES['gls_entry']} $tWhereClause ORDER BY abbreviation LIMIT $offset, $limit";
$lCountSQL = "SELECT count(*) AS count from {$_TABLES['gls_entry']} $tWhereClause";
$result = DB_query($lSQL);
$nrows  = DB_numRows($result);

// If any entries were found...
if ( $nrows > 0 ) {
    $T = new Template($_CONF['path'] . 'plugins/glossary/templates');
    if ($nrows < $_GLS_CONF['threshold']) {
        $T->set_file('entry','entry.thtml');
    } else {
        $T->set_file('entry','entry_list.thtml');
    }
    for ($i = 1; $i<=$nrows; $i++) {
        $bArray = DB_fetchArray($result);
        // Output each entry with template
        $T->set_var('abbreviation', $bArray['abbreviation']);
        // ignore newlines follwing html tags
        $lExplanation = nl2br(preg_replace('/\>\n/', '>', $bArray['explanation']));
        $lExplanation = GLS_insertGlossaryLinks($lExplanation, $bArray['abbreviation'], $bArray['project']);
        $T->set_var('explanation', $lExplanation);
        $lExplanation = GLS_truncate($bArray['explanation'], $bArray['abbreviation']);
        $lExplanation = GLS_insertGlossaryLinks($lExplanation, $bArray['abbreviation'], $bArray['project']);
        $T->set_var('short_explanation', $lExplanation);
        $projectName  = $projects["{$bArray['project']}"];
        $categoryName = $categories["{$bArray['category']}"];
        $T->set_var('reference', "[{$projectName}/{$categoryName}]");

        // View link (to detail/solo view)
        $T->set_var('view_link_label', $LANG_GLS_00['detail_view']);
        $T->set_var('view_link', $_CONF['site_url'] .'/glossary/entry.php?mode=view&id='. $bArray['id']);
        
        $T->set_var('category_label', $LANG_GLS_00['category']);
        $T->set_var('category', '<a href="index.php?category='. $bArray['category'] .'">'. $categories[$bArray['category']] .'</a>');
        
        $T->set_var('project_label', $LANG_GLS_00['project']);
        $T->set_var('project', '<a href="index.php?project='. $bArray['project'] .'">'. $projects[$bArray['project']] .'</a>');
        
        $T->set_var('owner_label', $LANG_GLS_00['owned_by']);
        $T->set_var('owner', '<a href="/users.php?mode=profile&uid='.$bArray['owner'].'">'.$users[$bArray['owner']].'</a>');
        
        $bCurTime = COM_getUserDateTimeFormat($bArray['timestamp']);
        $T->set_var('timestamp_label', $LANG_GLS_00['creation_on'] );
        $T->set_var('timestamp', $bCurTime[0] );
        if ( isset($bArray['last_owner']) && $bArray['last_owner']!=$bArray['owner'] ) {
            $T->set_var('last_owner_label', $LANG_GLS_00['last_owner']);
            $T->set_var('last_owner', '<a href="/users.php?mode=profile&uid='.$bArray['last_owner'].'">'.$users[$bArray['last_owner']].'</a>');
        }
        if ( isset($bArray['last_timestamp']) && $bArray['last_timestamp']!=$bArray['timestamp'] ) {
            $bCurTime = COM_getUserDateTimeFormat($bArray['last_timestamp']);
            $T->set_var('last_timestamp_label', $LANG_GLS_00['last_edit_on'] );
            $T->set_var('last_timestamp', $bCurTime[0] );
        }
        if ( SEC_hasRights('glossary.edit,glossary.admin','OR') ) {
            $T->set_var('edit_entry', '<a href="entry.php?mode=edit&id='.$bArray['id'].'">'. $LANG_GLS_00['edit_entry'] .'</a>');
        }
        if ( SEC_hasRights('glossary.admin') ) {
            $T->set_var('delete_entry_linkstart', '<a href="entry.php?mode='.$LANG24[11].'&id='.$bArray['id'].'">');
            $T->set_var('delete_entry', $LANG24[11]);
            $T->set_var('delete_entry_linkend', '</a>');
        }
        $T->parse('entry_list','entry', true);
    }
    $display .= $T->finish($T->get_var('entry_list'));
    
    
    // Show navigation bar for other pages
    $lCountData = DB_query($lCountSQL);
    $C = DB_fetchArray($lCountData);
    $lNumPages = ceil($C['count'] / $limit);

    // Print Google-like paging navigation
    $lURLParameters = GLS_assembleUrlParameters($selection, true);
    $lBaseURL  = $_CONF['site_url'] . '/glossary/index.php' . $lURLParameters;
    $display .= COM_printPageNavigation($lBaseURL, $page, $lNumPages);
}
$display  = GLS_render($display);
COM_output($display);
?>